<?php
/**
 * Copyright © Magento, Inc. All rights reserved.
 * See COPYING.txt for license details.
 */
namespace QuiversBopis\Integration\Block\Cart;

use Magento\Framework\App\Config\ScopeConfigInterface;
use Magento\Shipping\Model\Config;
use QuiversBopis\Integration\Model;
use QuiversBopis\Integration\Model\GetShippingMethod;
/**
 * @api
 * @SuppressWarnings(PHPMD.CouplingBetweenObjects)
 * @since 100.0.2
 */
class QuiversShipping  extends \Magento\Framework\View\Element\Template
{
    protected $scopeConfig;
    protected $shippingmodelconfig;  
    public $configProvider;
    protected $quote;
    protected $getshippingmodel;
    const XML_QUIVERS_UPC = 'quivers/parameters/quivers_custom_upc';
 
    public function __construct( \Magento\Directory\Model\Currency $currency,
    \QuiversBopis\Integration\Model\ConfigProvider $configProvider,
    \Magento\Checkout\Model\Session $checkoutSession,
    GetShippingMethod $getshippingmodel,
     Config $shippingmodelconfig, ScopeConfigInterface $scopeConfig){
        $this->shippingmodelconfig = $shippingmodelconfig;
        $this->scopeConfig = $scopeConfig;
        $this->quote = $checkoutSession->getQuote();
        $this->_currency = $currency;  
        $this->getshippingmodel =$getshippingmodel;  
        $this->configProvider = $configProvider; 
    }
    public function getUPC($product) {
        $scopeConfig = \Magento\Framework\App\ObjectManager::getInstance()->get(\Magento\Framework\App\Config\ScopeConfigInterface::class);
        $storeScope = \Magento\Store\Model\ScopeInterface::SCOPE_STORE;
        $quivers_upc = $scopeConfig->getValue(self::XML_QUIVERS_UPC, $storeScope);
        if ($quivers_upc) {
            return  $product->getData($quivers_upc);
        }else {
            return  $product->getData('upc');
        }
    }

    public function getModuleStatus () {  
        return $this->configProvider->getBopisStatus();
    }

    public function getHosotedJsPath() {
        return  $this->configProvider->getHosotedJsPath();
     }
 
    public function getMarketplace()
    {       
        return $this->configProvider->getMarketPlaceForCurrency(); 
    } 

    /**
     * Retrieve checkout configuration
     *
     * @return array
     * @codeCoverageIgnore
     */
    public function getShippingMethods($storeid) {
        
        return $this->getshippingmodel->getShippingMethods($storeid);    
   }

   public function getModalData($refid,$searchTerm,$storeFrontId,$postalcode) {
    $stores = array();
        try {
            $url= $this->configProvider->getApiBaseUrl().'v1/inventory/getFulfillmentOptions?storefrontId='.$storeFrontId.'&searchTerm='.$searchTerm;
            $response = $this->CallAPI(
                "GET",
                $url
            );
            if($response['httpcode'] == 200){
                $resultdata =json_decode($response['data'], true);
                return json_encode($resultdata['result']);
            }else {
                if($response['httpcode']==500 || $response['httpcode']==502) {
                    throw new \Exception("Internal server error");
                } else {
                    $message = isset(json_decode($response['data'], true)['error']['message'])?json_decode($response['data'], true)['error']['message']:json_decode($response['data'], true)['error'];
            
                    throw new \Exception($message);
               }
               
            }
        }
        catch(Exception $e) {
            $this->messageManager->addError(__($e->getMessage()));
        }
  }

  public function getSiteBaseUrl()
  {
    $objectManager = \Magento\Framework\App\ObjectManager::getInstance();
    $storeManager = $objectManager->get('\Magento\Store\Model\StoreManagerInterface');
    return  $storeManager->getStore()->getBaseUrl();
  }

  public function getApibaseUrl()
  {
      return $this->configProvider->getApiBaseUrl();
  }

  public function getcurrency()
  {
      return $this->configProvider->getCurrentCurrencySymbol();
  }

  Public function CallAPI($method, $url, $data = false, $headers = false, $timeout = 10)
  {
      $curl = curl_init();

      switch ($method)
      {
          case "POST":
              curl_setopt($curl, CURLOPT_POST, 1);
              if ($data)
                  curl_setopt($curl, CURLOPT_POSTFIELDS, $data);
              break;
          case "PUT":
              curl_setopt($curl, CURLOPT_PUT, 1);
              break;
          default:
              if ($data)
                  $url = sprintf("%s?%s", $url, http_build_query($data));
      }

      curl_setopt($curl, CURLOPT_URL, $url);
      curl_setopt($curl, CURLOPT_RETURNTRANSFER, 1);
      curl_setopt($curl, CURLOPT_CONNECTTIMEOUT, $timeout - 1);
      curl_setopt($curl, CURLOPT_TIMEOUT, $timeout);
      if ($headers) {
          foreach ($headers as $header) {
              curl_setopt( $curl, CURLOPT_HTTPHEADER, $header);
          }
      }

      $res = [];
      try{
          $result = curl_exec($curl);
      }
      catch(Exception $e) {
          return $res;
      }

      $httpcode = curl_getinfo($curl, CURLINFO_HTTP_CODE);
      $res['data'] = $result;
      $res['httpcode'] = @$httpcode;
      return $res;
  }


  
}
