<?php
namespace QuiversBopis\Integration\Block\Catalog\Product\View;

use QuiversBopis\Integration\Model;
use Magento\Catalog\Api\ProductRepositoryInterface;
use Magento\Framework\Exception\StateException;
use QuiversBopis\Integration\LogTracking;


class PickItUp extends \Magento\Framework\View\Element\Template
{
    /**
     * @var \Magento\Framework\Stdlib\ArrayUtils
     */
    protected $arrayUtils;
    public $configProvider;
    protected $_registry;
    protected $_product = null;
    protected $_storeManager;
    protected $scopeConfig;
    protected $messageManager;
    protected $productRepository;
    private $logger; 
    const XML_QUIVERS_UPC = 'quivers/parameters/quivers_custom_upc';

    /**
     * @param \Magento\Catalog\Block\Product\Context $context
     * @param \Magento\Framework\Stdlib\ArrayUtils $arrayUtils
     * @param array $data
     */
    public function __construct(
        \Magento\Catalog\Block\Product\Context $context,
        ProductRepositoryInterface $productRepository,
        \QuiversBopis\Integration\Model\ConfigProvider $configProvider,
        \Magento\Store\Model\StoreManagerInterface $storeManager,
        \Magento\Directory\Model\Currency $currency,
        \Magento\Framework\Registry $registry,
        \Magento\Framework\Message\ManagerInterface $messageManager,
        \Magento\Framework\Stdlib\ArrayUtils $arrayUtils,
        \Magento\Quote\Model\Quote $quote,
        \Magento\Catalog\Model\ProductFactory $productFactory,
        array $data = []
    ) {
        $this->configProvider = $configProvider;
        $this->arrayUtils = $arrayUtils;
        $this->_storeManager = $storeManager;
        $scopeConfig = \Magento\Framework\App\ObjectManager::getInstance()->get(\Magento\Framework\App\Config\ScopeConfigInterface::class);
        // $log = new LogTracking($scopeConfig);
        $this->_currency = $currency;  
        $this->_registry = $registry;
        $this->messageManager = $messageManager;
        $this->_productFactory = $productFactory;
        $this->productRepository = $productRepository;
        parent::__construct(
            $context,
            $data
        );
    }

    public function getModuleStatus () {  
        return $this->configProvider->getBopisStatus();
    }

    public function getProduct()
    {
        $product= $this->_registry->registry('current_product');
        if ($product && $product->getTypeInstance()->getStoreFilter($product) === null) {
            $product->getTypeInstance()->setStoreFilter($this->_storeManager->getStore(), $product);
        }
        return $product;
    }

    public function getSiteBaseUrl()
    {
        $objectManager = \Magento\Framework\App\ObjectManager::getInstance();
        $storeManager = $objectManager->get('\Magento\Store\Model\StoreManagerInterface');
        return  $storeManager->getStore()->getBaseUrl();
    }

    public function getApibaseUrl()
    {
        return $this->configProvider->getApiBaseUrl();
    }

    public function getCurrentCategory()
    {        
        return $this->_registry->registry('current_category');
    }
    
    public function getCurrentProduct()
    {       
        return $this->_registry->registry('current_product');
    } 

    public function getUPC() {
        $scopeConfig = \Magento\Framework\App\ObjectManager::getInstance()->get(\Magento\Framework\App\Config\ScopeConfigInterface::class);
        $storeScope = \Magento\Store\Model\ScopeInterface::SCOPE_STORE;
        $quivers_upc = $scopeConfig->getValue(self::XML_QUIVERS_UPC, $storeScope);
        $product = $this->getCurrentProduct();
        if ($quivers_upc) {
            return  $product->getData($quivers_upc);
        }else {
            return  $product->getData('upc');
        }
    }

    public function getSku()
    {       
        return $this->getCurrentProduct()->getSku();
    }  

    public function getMarketplace()
    {       
        return $this->configProvider->getMarketPlaceForCurrency(); 
    } 

    public function getApiKey()
    {       
        return $this->configProvider->getQuiversApiKey();   
    } 


    public function getSearchTerm()
    {       
            $sku =  $this->getSku();
            $upc = $this->getUPC();
            if (empty($sku) && empty($upc) && $this->configProvider->getBopisStatus()==true) {
                throw new \Exception('Not Able to fetch SKU or UPC for this Product.');
            }else{
               
                return ['sku'=>$sku,'upc'=>$upc];
            }

    }  

    public function getHosotedJsPath() {
       return  $this->configProvider->getHosotedJsPath();
	}

    public function getConfigProdskus(){
        $scopeConfig = \Magento\Framework\App\ObjectManager::getInstance()->get(\Magento\Framework\App\Config\ScopeConfigInterface::class);
        $storeScope = \Magento\Store\Model\ScopeInterface::SCOPE_STORE;
        $quivers_upc = $scopeConfig->getValue(self::XML_QUIVERS_UPC, $storeScope);

        $skus = [];
        $prod = $this->_registry->registry('current_product');
        if($prod->getTypeId() == "configurable"){
            $children = $prod->getTypeInstance()->getUsedProducts($prod);
            if(!empty($children)){
                foreach ($children as $child){
                    $childData = $child->getData();
                    $childId =$childData["entity_id"];
                    if ($quivers_upc && $child->getData($quivers_upc)) {
                        $skus[$childId] = $child->getData($quivers_upc);
                        
                    }elseif ($child->getData('upc')){
                        $skus[$childId] = $child->getData('upc');
                    }
                    else{
                        $skus[$childId] = $child->getSku();

                    }
                }
            }  
        }
        return $skus;
    }

    public function isSaleable()
    { 
        $storeFrontId = $this->getMarketplace();
        $searchTerm = $this->getSearchTerm();
        $searchkey =null;
        $postalcodes = [];
        if ($this->getModuleStatus() && isset($storeFrontId)) {
            try {
                if($searchTerm['upc']==null){
                    $searchkey =$searchTerm['sku'];
                   $url= $this->configProvider->getApiBaseUrl().'v1/inventory/getFulfillmentOptions?storefrontId='.$storeFrontId.'&searchTerm='.$searchTerm['sku'];
                }else{
                    $searchkey =$searchTerm['upc'];
                    $url=$this->configProvider->getApiBaseUrl().'v1/inventory/getFulfillmentOptions?storefrontId='.$storeFrontId.'&searchTerm='.$searchTerm['upc'];
                }
                $response = $this->CallAPI(
                    "GET",
                    $url
                );
                if($response['httpcode'] == 200){
                    $resultdata =json_decode($response['data'], true);
                    if(isset($searchTerm['sku']) && $searchTerm['upc']!=null &&  isset($resultdata['result']) && empty($resultdata['result'])){
                        $searchkey =$searchTerm['sku'];
                        $url= $this->configProvider->getApiBaseUrl().'v1/inventory/getFulfillmentOptions?storefrontId='.$storeFrontId.'&searchTerm='.$searchTerm['sku'];
                        $response = $this->CallAPI(
                            "GET",
                            $url
                        );
                        if($response['httpcode'] == 200){
                            foreach ($resultdata['result'] as $store ) {
                                array_push($postalcodes,$store['address']['postalCode']);
                            }
                            $result =[
                                'postalcodes'=>json_encode($postalcodes),
                                'stores'=>$resultdata['result'],
                                'searchterm'=>$searchkey
                            ];
                            return $result;
                        }
                    }else{
                        foreach ($resultdata['result'] as $store ) {
                            array_push($postalcodes,$store['address']['postalCode']);
                        }
                        $result =[
                            'postalcodes'=>json_encode($postalcodes),
                            'stores'=>$resultdata['result'],
                            'searchterm'=>$searchkey
                        ];
                        return $result;
                    }
                  
         
                }else {
                                     
                       return  $result =[
                            'postalcodes'=>null,
                            'stores'=>null,
                            'searchterm'=>$searchkey
                        ];
                   
                }
            }
            catch(Exception $e) {
                $this->messageManager->addError(__($e->getMessage()));
            }
        }else {
            $request_data=[
                'storefrontId'=>null,
                'stores'=>null,
                'searchterm'=>null

            ];
        }
        
     
    }

    Public function getIPAddress() {  
        if(isset($_SERVER['REMOTE_ADDR'])) 
        {
            $ipaddress = $_SERVER['REMOTE_ADDR'];
            return $ipaddress;
        }
        $ip_address=file_get_contents('http://checkip.dyndns.com/');
        $result = explode(": ", $ip_address);
        $result1 = explode("<", $result[1]);
        $ip = $result1[0]; // piece1
        return $ip;  
    } 

    Public function CallAPI($method, $url, $data = false, $headers = false, $timeout = 10)
	{
		$curl = curl_init();

		switch ($method)
		{
			case "POST":
				curl_setopt($curl, CURLOPT_POST, 1);
				if ($data)
					curl_setopt($curl, CURLOPT_POSTFIELDS, $data);
				break;
			case "PUT":
				curl_setopt($curl, CURLOPT_PUT, 1);
				break;
			default:
				if ($data)
					$url = sprintf("%s?%s", $url, http_build_query($data));
		}

		curl_setopt($curl, CURLOPT_URL, $url);
		curl_setopt($curl, CURLOPT_RETURNTRANSFER, 1);
		curl_setopt($curl, CURLOPT_CONNECTTIMEOUT, $timeout - 1);
		curl_setopt($curl, CURLOPT_TIMEOUT, $timeout);
		if ($headers) {
			foreach ($headers as $header) {
				curl_setopt( $curl, CURLOPT_HTTPHEADER, $header);
			}
		}

		$res = [];
		try{
			$result = curl_exec($curl);
		}
		catch(Exception $e) {
			return $res;
		}

		$httpcode = curl_getinfo($curl, CURLINFO_HTTP_CODE);
		$res['data'] = $result;
		$res['httpcode'] = @$httpcode;
		return $res;
	}


    
}
