<?php

namespace QuiversBopis\Integration\Block\Email;

use Magento\Catalog\Api\ProductRepositoryInterface;
use Magento\Catalog\Block\Product\Context;
use Magento\Catalog\Model\Product;
use Magento\Directory\Model\PriceCurrency;
use Magento\Framework\Exception\NoSuchEntityException;
use Magento\Framework\UrlInterface;
use Magento\Framework\View\Element\Template;
use Magento\Quote\Model\Quote;
use Magento\Quote\Model\Quote\Item;
use Magento\Quote\Model\QuoteFactory;
use Magento\Store\Model\Store;
use Magento\Tax\Helper\Data;
use Magento\Tax\Model\Config;

/**
 * Class CartItems
 * @package QuiversBopis\Integration\Block\Email
 */
class CartItems extends Template
{
    /**
     * @var ProductRepositoryInterface
     */
    protected $_productRepository;

    /**
     * @var PriceCurrency
     */
    protected $priceCurrency;

    /**
     * @var Data
     */
    protected $taxHelper;

    /**
     * @var QuoteFactory
     */
    protected $quoteFactory;


    /**
     * CartItems constructor.
     *
     * @param Context $context
     * @param ProductRepositoryInterface $productRepository
     * @param PriceCurrency $priceCurrency
     * @param QuoteFactory $quoteFactory
     * @param array $data
     */
    public function __construct(
        Context $context,
        ProductRepositoryInterface $productRepository,
        \Magento\Sales\Api\OrderRepositoryInterface $orderRepository,
        PriceCurrency $priceCurrency,
        QuoteFactory $quoteFactory,
        array $data = []
    ) {
        $this->_productRepository = $productRepository;
        $this->orderRepository = $orderRepository;
        $this->priceCurrency = $priceCurrency;
        $this->taxHelper = $context->getTaxData();
        $this->quoteFactory = $quoteFactory;
        parent::__construct($context, $data);
    }


     /**
     * @return array
     */
    public function getItemOptions($quoteItem)
    {
        $result = [];
        if ($options = $quoteItem->getProduct()->getTypeInstance(true)->getOrderOptions($quoteItem->getProduct())) {
            if (isset($options['options'])) {
                $result = array_merge($result, $options['options']);
            }
            if (isset($options['additional_options'])) {
                $result = array_merge($result, $options['additional_options']);
            }
            if (isset($options['attributes_info'])) {
                $result = array_merge($result, $options['attributes_info']);
            }
        }
        return $result;
    }

    /**
     * @return Quote|null
     */
    public function getQuote()
    {
        
        try {
            $orderid =$this->getData('order');
            $objectManager = \Magento\Framework\App\ObjectManager::getInstance();
            $collection = $objectManager->create('Magento\Sales\Model\Order'); 
            $orderInfo = $collection->loadByIncrementId($orderid);
         } catch (NoSuchEntityException $e) {
            throw new \Magento\Framework\Exception\LocalizedException(__('This order no longer exists.'));
         }
        $quoteId = $orderInfo->getQuoteId();
        if ($quoteId) {
            return $this->quoteFactory->create()->load($quoteId);
        }

        return null;
    }

   /**
     * @return number
    */
    public function getShippingStatus()
    {
        $shipping = 0;
        $instore = 0;
        $quote = $this->getQuote();
        if ($quote == null){
            return 0;
        }
        foreach($quote->getAllItems() as $quoteItem){
            $itemData = $quoteItem->getData();
            if(array_key_exists('parent_item_id', $itemData) 
                && !is_null($itemData['parent_item_id'])) {
                continue;
            }
            $AdditionalData =$quoteItem->getBopisdata();
            $AdditionalData=json_decode($AdditionalData);
            if(isset($AdditionalData->businessRefId)){
                $instore =1;
            }
            else{
                $shipping =1;
            }

        }
        if($shipping==1 && $instore==1 ){
            return 2;
        }else if($shipping==0 && $instore==1){
            return 1;
        }
        else{
            return 0;
        }

    }

    public function getOrderInfo()
    { 
        try 
        {
            $orderid =$this->getData('order');
            $objectManager = \Magento\Framework\App\ObjectManager::getInstance();
            $collection = $objectManager->create('Magento\Sales\Model\Order'); 
            $orderInfo = $collection->loadByIncrementId($orderid);
            return $orderInfo;            
        } catch (NoSuchEntityException $e) {
            throw new \Magento\Framework\Exception\LocalizedException(__('This order no longer exists.'));
            return null;
        }
    }



    /**
     * Get totals in quote
     *
     * @return total[]
     */
    public function getTotals()
    {
        $source = $this->getOrderInfo();
        $_totals = [];

        /**
         * Add subtotal
         */
        $_totals['subtotal'] = new \Magento\Framework\DataObject(
            ['code' => 'subtotal', 'value' => $source->getSubtotal(), 'label' => __('Subtotal')]
        );

        /**
         * Add discount
         */
        if ((double)$source->getDiscountAmount() != 0) {
            if ($source->getDiscountDescription()) {
                $discountLabel = __('Discount (%1)', $source->getDiscountDescription());
            } else {
                $discountLabel = __('Discount');
            }
            $_totals['discount'] = new \Magento\Framework\DataObject(
                [
                    'code' => 'discount',
                    'field' => 'discount_amount',
                    'value' => $source->getDiscountAmount(),
                    'label' => $discountLabel,
                ]
            );
        }
        /**
         * Add shipping
         */
        if (!$source->getIsVirtual() && ((double)$source->getShippingAmount() || $source->getShippingDescription())) {
            $label = __('Shipping & Handling');
            if ($source->getCouponCode() && !isset($_totals['discount'])) {
                $label = __('Shipping & Handling (%1)', $source->getCouponCode());
            }

            $_totals['shipping'] = new \Magento\Framework\DataObject(
                [
                    'code' => 'shipping',
                    'field' => 'shipping_amount',
                    'value' => $source->getShippingAmount(),
                    'label' => $label,
                ]
            );
        }

        /**
         * Add Tax
         */
        if ($source->getTaxAmount() > 0){
            $_totals['tax'] = new \Magento\Framework\DataObject(
                [
                    'code' => 'tax',
                    'field' => 'tax',
                    'value' => $source->getTaxAmount(),
                    'label' => __('Estimated Tax'),
                ]
            );
    
        }

        /**
         * Add Included Tax
         */
        if ($source->getQuiversIncludedTax() > 0){
            $_totals['included_tax'] = new \Magento\Framework\DataObject(
                [
                    'code' => 'included_tax',
                    'field' => 'included_tax',
                    'value' => $source->getQuiversIncludedTax(),
                    'label' => __('Estimated Included Tax'),
                ]
            );
    
        }
        /**
         * Add Grand total
         */
        $_totals['grand_total'] = new \Magento\Framework\DataObject(
            [
                'code' => 'grand_total',
                'field' => 'grand_total',
                'strong' => true,
                'value' => $source->getGrandTotal(),
                'label' => __('Grand Total'),
            ]
        );

        /**
         * Base grandtotal
         */

        if ($source->isCurrencyDifferent()) {
            $_totals['base_grandtotal'] = new \Magento\Framework\DataObject(
                [
                    'code' => 'base_grandtotal',
                    'value' => $source->formatBasePrice($source->getBaseGrandTotal()),
                    'label' => __('Grand Total to be Charged'),
                    'is_formated' => true,
                ]
            );
        }
        return $_totals;
    }


    /**
     * Get items in quote
     *
     * @return Item[]
     */
    public function getProductCollection()
    {
        $items = [];
        if ($quote = $this->getQuote()) {
            return $quote->getAllItems();
        }
        return $items;
    }


    /**
     * @param int|float $value
     *
     * @return float|string
     */
    public function formatPrice($value)
    {
        $quote = $this->getQuote();
        $storeId = $quote ? $quote->getStoreId() : null;
        return $this->priceCurrency->format(
            $value,
            true,
            PriceCurrency::DEFAULT_PRECISION,
            $storeId
        );
    }

    /**
     * @return Config
     */
    public function getTaxConfig()
    {
        return $this->taxHelper->getConfig();
    }
}
