<?php
/**
* Copyright © Magento, Inc. All rights reserved.
* See COPYING.txt for license details.
*/
namespace QuiversBopis\Integration\Model\Quote;
 
use Magento\Quote\Model\Quote\Item\AbstractItem;
use QuiversBopis\Integration\Model\ConfigProvider;
use Magento\Store\Model\ScopeInterface;
 
class Address extends \Magento\Quote\Model\Quote\Address
{
   public $configProvider;
   public $storeManager;
   public $CcConfig;
   public $i=0;
       /**
    * Retrieve item quantity by id
    *
    * @param int $itemId
    * @return float|int
    */
   public function getItemQty($itemId = 0)
   {
        $objectManager = \Magento\Framework\App\ObjectManager::getInstance();
            
        $CcConfig =$objectManager->get(\Magento\Payment\Model\CcConfig::class);
        $this->storeManager = $this->storeManager ?: $objectManager->get(\Magento\Store\Model\StoreManagerInterface::class);
        $this->configProvider = new ConfigProvider($CcConfig,$this->_scopeConfig,$this->storeManager);
        if($this->configProvider->getBopisStatus()==false){
            $qty =  parent::getItemQty($itemId);
            return $qty;
        }
 
        $quote_items =$this->_quote->getAllItems();
        $qty = 0;
        foreach ($quote_items as $quoteitem) {
            $itemData = $quoteitem->getData();
            if(array_key_exists('parent_item_id', $itemData) 
                    && $itemData['parent_item_id'] != null) {
                continue;
            }
            $bopisdata = "";
            if (array_key_exists('bopisdata', $itemData)){
                $bopisdata = $itemData['bopisdata'];
                if ($bopisdata != ""){
                    $bopisdata = json_decode($bopisdata);
                }
            }
            if($itemId == 0 ){
                if ($bopisdata==""){
                    $qty = $qty+(int)$quoteitem->getQty();
                }
                elseif(isset($bopisdata->curbsidepick) && $bopisdata->curbsidepick==false  &&
                       isset($bopisdata->instorepickup) && $bopisdata->instorepickup==false){
                        $qty = $qty+(int)$quoteitem->getQty();
                 }
            }
            if($itemId == $quoteitem->getId()){
                if ($bopisdata==""){
                    $item = $this->getItemById($itemId);
                    if ($item) {
                        $qty = $item->getQty();
                    }

                }
                elseif( isset($bopisdata->curbsidepick) && $bopisdata->curbsidepick==false  &&
                        isset($bopisdata->instorepickup) && $bopisdata->instorepickup==false){
                    $item = $this->getItemById($itemId);
                    if ($item) {
                        $qty = $item->getQty();
                    } 
                }
            }
        }
        return $qty;
   }
 
    public function requestShippingRates(AbstractItem $item = null)
    {
        $objectManager = \Magento\Framework\App\ObjectManager::getInstance();
      
        $CcConfig = $objectManager->get(\Magento\Payment\Model\CcConfig::class);
        $this->storeManager = $this->storeManager ?: $objectManager->get(\Magento\Store\Model\StoreManagerInterface::class);
        $this->configProvider = new ConfigProvider($CcConfig,$this->_scopeConfig,$this->storeManager);
        if($this->configProvider->getBopisStatus()==false){
            $found =   parent::requestShippingRates($item);
            return $found;
        }
        $quote_items =$this->_quote->getAllItems();
        $allshippingRates =[];
        $storeId = $this->getQuote()->getStoreId() ?: $this->storeManager->getStore()->getId();
        $taxInclude = $this->_scopeConfig->getValue(
            'tax/calculation/price_includes_tax',
            ScopeInterface::SCOPE_STORE,
            $storeId
        );
 
        /** @var $request RateRequest */
        $request = $this->_rateRequestFactory->create();
        $request->setAllItems($item ? [$item] : $this->getAllItems());
        $request->setDestCountryId($this->getCountryId());
        $request->setDestRegionId($this->getRegionId());
        $request->setDestRegionCode($this->getRegionCode());
        $request->setDestStreet($this->getStreetFull());
        $request->setDestCity($this->getCity());
        $request->setDestPostcode($this->getPostcode());
        $baseSubtotal = $taxInclude ? $this->getBaseSubtotalTotalInclTax() : $this->getBaseSubtotal();
        $request->setPackageValue($item ? $item->getBaseRowTotal() : $baseSubtotal);
        $baseSubtotalWithDiscount = $baseSubtotal + $this->getBaseDiscountAmount();
        $packageWithDiscount = $item ? $item->getBaseRowTotal() -
            $item->getBaseDiscountAmount() : $baseSubtotalWithDiscount;
        $request->setPackageValueWithDiscount($packageWithDiscount);
        foreach ($quote_items as $quoteitem) {
            $bopisdata = json_decode($quoteitem->getBopisdata());
            $itemData = $quoteitem->getData();
            if(array_key_exists('parent_item_id', $itemData) 
                    && $itemData['parent_item_id'] != null) {
                continue;
            }

            $bopisdata = "";
            if (array_key_exists('bopisdata', $itemData)){
                $bopisdata = $itemData['bopisdata'];
                if ($bopisdata != ""){
                    $bopisdata = json_decode($bopisdata);
                }
            }

            if($item == null || $item->getId() == $quoteitem->getId()){
                if(isset($bopisdata->curbsidepick) && $bopisdata->curbsidepick==true  && isset($bopisdata->instorepickup) && $bopisdata->instorepickup==true){
                    $request->setPackageWeight(0);
                    $request->setPackageQty($this->getItemQty());
                    /**
                        * Need for shipping methods that use insurance based on price of physical products
                        */
                    $packagePhysicalValue =$baseSubtotal - 0;
                    $request->setPackagePhysicalValue($packagePhysicalValue);
                    $request->setFreeMethodWeight(0);
                }else{
                    
                    $request->setPackageWeight($item ? $item->getRowWeight() : $this->getWeight());
                    $request->setPackageQty($item ? $item->getQty() : $this->getItemQty());
                    /**
                        * Need for shipping methods that use insurance based on price of physical products
                        */
                    $packagePhysicalValue = $item ? $item->getBaseRowTotal() : $baseSubtotal - $this->getBaseVirtualAmount();
                    $request->setPackagePhysicalValue($packagePhysicalValue);
            
                    $request->setFreeMethodWeight($item ? 0 : $this->getFreeMethodWeight());
                }
            }
        }
 
        /**
            * Store and website identifiers specified from StoreManager
            */
        $request->setStoreId($storeId);
        if ($this->getQuote()->getStoreId()) {
            $request->setWebsiteId($this->storeManager->getStore($storeId)->getWebsiteId());
        } else {
            $request->setWebsiteId($this->storeManager->getWebsite()->getId());
        }
        $request->setFreeShipping($this->getFreeShipping());
        /**
            * Currencies need to convert in free shipping
            */
        $request->setBaseCurrency($this->storeManager->getStore()->getBaseCurrency());
        $request->setPackageCurrency($this->storeManager->getStore()->getCurrentCurrency());
        $request->setLimitCarrier($this->getLimitCarrier());
        $baseSubtotalInclTax = $this->getBaseSubtotalTotalInclTax();
        $request->setBaseSubtotalInclTax($baseSubtotalInclTax);
    
        $result = $this->_rateCollector->create()->collectRates($request)->getResult();
    
        $found = false;
        if ($result) {
            $shippingRates = $result->getAllRates();
    
            foreach ($shippingRates as $shippingRate) {
                $rate = $this->_addressRateFactory->create()->importShippingRate($shippingRate);
                if (!$item) {
                    $this->addShippingRate($rate);
                }

                if ($this->getShippingMethod() == $rate->getCode()) {
                    if ($item) {
                        $item->setBaseShippingAmount($rate->getPrice());
                    } else {
    
                        /** @var StoreInterface */
                        $store = $this->storeManager->getStore();
                        $amountPrice = $store->getBaseCurrency()
                            ->convert($rate->getPrice(), $store->getCurrentCurrencyCode());
                        $this->setBaseShippingAmount($rate->getPrice());
                        $this->setShippingAmount($amountPrice);
                    }
    
                    $found = true;
                }
            }
        }
 
        // custom code of Quivers
        $objectManager = \Magento\Framework\App\ObjectManager::getInstance();
        $checkoutSession = $objectManager->get('Magento\Checkout\Model\Session');
        $ConfigProvider = $objectManager->get('QuiversBopis\Integration\Model\ConfigProvider');
        $bopismixed =$ConfigProvider->bopismixed;
        $storeid =$checkoutSession->getQuiversBopisStoreid();
        $bopispickup =$ConfigProvider->storepickupcode;
        $final_methods =[];
        $quivers_methods = [];
        if ($result) {
                $shippingRates = $result->getAllRates();
        }else{
            $shippingRates =[];
        }

        foreach ($shippingRates as $shippingRate) {
            $rate = $this->_addressRateFactory->create()->importShippingRate($shippingRate);
            $bopismixedcode =$bopismixed;
            $bopispickupcode =$bopispickup;
            $carrierCode =$rate->getCode();
            $title = $rate->getCarrierTitle();;
            $price = $rate->getPrice()==null?0:$rate->getPrice();

            $carrierCode = explode("_", $carrierCode)[0];
            if ($carrierCode!= $bopismixedcode)
            {
                if ($carrierCode!=$bopispickupcode){
                    
                    $quivers_methods[] = array('code'=>$carrierCode,'label'=>$title,'price'=>$price);
                }
                if($storeid!=null && $carrierCode==$bopispickupcode){
                    $final_methods[] = array('code'=>$carrierCode,'label'=>$title,'price'=>$price);
                }
                if($storeid==null && $carrierCode!=$bopispickupcode){
                    $final_methods[] = array('code'=>$carrierCode,'label'=>$title,'price'=>$price);
                }
            }
            array_push($allshippingRates,array('code'=>$carrierCode,'label'=>$title,'price'=>$price));
        }

        if(isset($final_methods[0])){
            $checkoutSession->setQuiversShippingRates($final_methods);
            $checkoutSession->setAllShippingRates($allshippingRates);
            if(!isset($_SESSION['quivers_shipping_rate']) || $_SESSION['quivers_shipping_rate']==[]){
                $_SESSION['quivers_shipping_rate'] = $quivers_methods[0];
            } else{
                    $session_method = isset($_SESSION['quivers_shipping_rate'])?$_SESSION['quivers_shipping_rate']:[];
                    foreach($quivers_methods as $quivers_method){
                        if ($quivers_method['code'] == $session_method['code']){
                            $_SESSION['quivers_shipping_rate'] =$quivers_method;
                            break;
                        }
                    }
                    if(!isset($_SESSION['quivers_shipping_rate']) || $_SESSION['quivers_shipping_rate']==[]){
                        $_SESSION['quivers_shipping_rate'] =$quivers_methods[0];
            
                    }        
            }
        }else{
            $_SESSION['quivers_shipping_rate'] =[];
            $checkoutSession->setQuiversShippingRates([]);
        }
        return $found;
    
    }
 }
