<?php
namespace QuiversBopis\Integration\Observer;

use Magento\Framework\Event\ObserverInterface;

class EmailAddCustomVariable implements ObserverInterface
{
    /**
     *
     * @param \Magento\Framework\Event\Observer $observer
     * @return void
     */
    public function execute(\Magento\Framework\Event\Observer $observer)
    {
        //Add variables to use in Quivers Custom Order Email Template
        $objectManager = \Magento\Framework\App\ObjectManager::getInstance();
        $checkoutSession = $objectManager->get('Magento\Checkout\Model\Session');
        $customerSession = $objectManager->get('Magento\Customer\Model\Session');
        $shippingStatus = $checkoutSession->getQuiversShippingStatus();
        /** @var \Magento\Framework\App\Action\Action $controller */
        $data = $observer->getEvent()->getData();
        $transportobject = $data["transportObject"];
        $transport = []; 
        $transport['quivers_is_logged_in'] = $customerSession->isLoggedIn();
        if ($shippingStatus == 2){
            $transport['quivers_shipping'] = false;
            $transport['quivers_bopismixed'] = true;
            $transport['quivers_bopis'] = false;
        }
        elseif ($shippingStatus == 1){
            $transport['quivers_shipping'] = false;
            $transport['quivers_bopismixed'] = false;
            $transport['quivers_bopis'] = true;
        }
        else{
            $transport['quivers_shipping'] = true;
            $transport['quivers_bopismixed'] = false;
            $transport['quivers_bopis'] = false;
        }
        $quote=  $checkoutSession->getQuote();
        $transportobjectData = $transportobject->getData(); 
        $EventOrder = $transportobjectData["order"];
        $order_data = $EventOrder->getData();
        $transport["quivers_customer_name"] =$this->getCustomerName($order_data);
        $transport["quivers_is_not_virtual"] = !$quote->getIsVirtual();
        $transport["quivers_email_customer_note"] =  $this->getEmailCustomerNote($quote);

        $transport["quivers_created_at_formatted"] = $this->getCreatedAtFormatted(2);
        $transportobject->addData($transport);
        

    }


    public function getCustomerName($order_data)
    {
        $addresses = $order_data["addresses"];
        foreach ($addresses as $address) {
            if ($address->getAddressType() == 'billing' && !$address->isDeleted()) {
                if (null === $address->getFirstname()) {
                    return (string)__('Guest');
                }

                $customerName = '';
                if (strlen($address->getPrefix())) {
                    $customerName .= $address->getPrefix() . ' ';
                }
                $customerName .= $address->getFirstname();
                if (strlen($address->getMiddlename())) {
                    $customerName .= ' ' . $address->getMiddlename();
                }
                $customerName .= ' ' . $address->getLastname();
                if (strlen($address->getSuffix())) {
                    $customerName .= ' ' . $address->getSuffix();
                }
                return $customerName;

            }
        }
        return "";

    }


    public function getEmailCustomerNote($quote)
    {

        if ($quote->getCustomerNoteNotify()) {
            return $quote->getCustomerNote();
        }
        return '';
    }

    public function getCreatedAtFormatted($format)
    {

        $objectManager = \Magento\Framework\App\ObjectManager::getInstance();
        $timezone = $objectManager->get('\Magento\Framework\Stdlib\DateTime\TimezoneInterface');
        $checkoutSession = $objectManager->get('Magento\Checkout\Model\Session');
        $_storeManager = $objectManager->get('Magento\Store\Model\StoreManagerInterface');
        $localeResolver = $objectManager->get('Magento\Framework\Locale\ResolverInterface');

        $quote=  $checkoutSession->getQuote();
        $storeId = $quote->getStoreId();
        if ($storeId) {
            $store = $_storeManager->getStore($storeId);
        }else{
            $store= $_storeManager->getStore();

        }
        return $timezone->formatDateTime(
            new \DateTime($quote->getCreatedAt()),
            $format,
            $format,
            $localeResolver->getDefaultLocale(),
            $timezone->getConfigTimezone('store', $store));
    }

}

