<?php
namespace QuiversBopis\Integration\Observer;
use  QuiversBopis\Integration\Model\ConfigProvider;
use QuiversBopis\Integration\LogTracking;

class OrderGenrateEvent implements \Magento\Framework\Event\ObserverInterface {
    /**
 * @var ObjectManagerInterface
 */
protected $_objectManager;
protected $configProvider;

/**
 * @param \Magento\Framework\ObjectManagerInterface $objectManager
 */
public function __construct(
    \Magento\Framework\ObjectManagerInterface $objectManager,
    \Magento\Quote\Model\QuoteFactory $quoteFactory,
    \Magento\Sales\Model\Order $order,
    ConfigProvider $configProvider
) {
    $this->_objectManager = $objectManager;
    $this->order = $order;
    $this->configProvider =$configProvider;
}

/**
 *
 * @param \Magento\Framework\Event\Observer $observer
 * @return void
 */
public function execute(\Magento\Framework\Event\Observer $observer)
{
    $objectManager = \Magento\Framework\App\ObjectManager::getInstance();
    $logTracking = $objectManager->get('\QuiversBopis\Integration\LogTracking');

    $order = $observer->getEvent()->getOrder();
    $orderId = $order->getIncrementId();

    $itemdata = [
        'inbound_order_id'=>$orderId,
        'storefront_id'=>$this->configProvider->getMarketPlaceForCurrency(),
        "items"=>json_encode($this->configProvider->getQuotedataToSetOrder())
    ];
    $data = [
        'key'=>$this->configProvider->getQuiversApiKey(),
        "value"=>json_encode($itemdata)
    ];

    $header = [array('Authorization:'."apikey ".$this->configProvider->getQuiversApiKey())];
    $url = $this->configProvider->getApiBaseUrl()."v1/storage/set?business=".$this->configProvider->getQuiversRefId();
    $logTracking->orderGenerate($url, $data, $header);
    try{
        $response = $this->configProvider->CallAPI(
            "POST",
            $url,
            json_encode($data),
            $header
        );
        $logTracking->orderGenerateSuccess($url, $data, $header, $response);
        $writer = new \Zend\Log\Writer\Stream(BP . '/var/log/debug.log');
        $logger = new \Zend\Log\Logger();
        $logger->addWriter($writer);
        $logger->info( "orderresponse".json_encode($response));
    }
    catch(\Exception $e)
    {
        throw new \Exception($e->getMessage());
    }
}
}