<?php
/**
 * Copyright © 2015 Magento. All rights reserved.
 * See COPYING.txt for license details.
 */
namespace QuiversBopis\Integration\Observer;

use Magento\Checkout\Model\Session as CheckoutSession;
use Magento\Framework\Event\ObserverInterface;

class SetShippingMethod implements ObserverInterface
{
   /**
     * Collect grand total address amount
     *
     * @param \Magento\Quote\Model\Quote $quote
     * @param \Magento\Quote\Api\Data\ShippingAssignmentInterface $shippingAssignment
     * @param \Magento\Quote\Model\Quote\Address\Total $total
     * @return $this
     */
    protected $quoteValidator = null; 
    private $checkoutSession;
    private $addressFactory;
    private $addressInformation;

    public function __construct(\Magento\Quote\Model\QuoteValidator $quoteValidator,
    \Magento\Sales\Model\Order\AddressRepository $addressRepo, 
    \Magento\Checkout\Api\Data\TotalsInformationInterface $addressInformation,  CheckoutSession $checkoutSession)
    {
        $this->quoteValidator = $quoteValidator;
        $this->checkoutSession = $checkoutSession;
        $this->_addressRepo = $addressRepo;
        $this->addressInformation = $addressInformation;
        
    }
    /**
     * Has shipping been applied to quote?
     *
     * @var bool
     */
    protected $_hasShipping = false;

    /**
     * Set shipping method and rate if they do not exist yet
     */
    public function execute(\Magento\Framework\Event\Observer $observer)
{
 
        $event = $observer->getEvent();
        $objectManager = \Magento\Framework\App\ObjectManager::getInstance();
        $checkoutsession = $objectManager->get('\Magento\Checkout\Model\Session');
        $total = $objectManager->get('\Magento\Quote\Model\Quote\Address\Total');
        $configProvider = $objectManager->get('\QuiversBopis\Integration\Model\ConfigProvider');

        $quote = $checkoutsession->getQuote();
        $items = $quote->getAllItems();
        $shippingFeesTotal = 0.0;
        $shipLabel = "";
        $shipping_address = $quote->getShippingAddress();
        $shippingMethod = $shipping_address->getShippingMethod();
        $shippingRateData = $shipping_address->getShippingRatesCollection()->getData();
        if(count($shippingRateData)>0) {
            foreach ($shippingRateData as $rate) { 
                if($shippingMethod == $rate['code']) {
                    $shippingFeesTotal =  $rate['price'] ;
                    $shipLabel = $rate['carrier_title'];
                }
            }
        }


        $totalShippingRate = $total->getShippingAmount();
        if ($shippingMethod != "" && $shippingFeesTotal <= 0){
            $shippingFeesTotal = $totalShippingRate;
        }
        if ($shippingMethod != "" && $shippingFeesTotal <= 0){
            $shippingAmount = $quote->getShippingAddress()->getShippingAmount();
            $shippingFeesTotal = $shippingAmount;
        }

        $shipped_qty = 0;
        foreach($items as $item){
            $itemData = $item->getData();
            if(array_key_exists('parent_item_id', $itemData) 
               && !is_null($itemData['parent_item_id'])) {
                continue;
            }
            $shipped_qty = $shipped_qty + 1;
        }

        $approx_ship_amounts = [];
        if ($checkoutsession->getQuiversShippingStatus() == 0){
            $approx_ship_amounts = $configProvider->splitShippingLineItem($shippingFeesTotal, $shipped_qty);
        }
        if ($shipLabel == ""){
            $carrierCode = explode("_", $shippingMethod)[0];
            $methods = $checkoutsession->getAllShippingRates();
            if ($methods != []){
                foreach ($methods as $rate) { 
                    if($carrierCode == $rate['code']) {
                        $shipLabel = $rate['label'];
                    }
                }
            }

        }

        $i= 0;

        foreach($items as $item){
            $itemData = $item->getData();
            if(array_key_exists('parent_item_id', $itemData) 
               && !is_null($itemData['parent_item_id'])) {
                continue;
            }
            if ($checkoutsession->getQuiversShippingStatus() == 0){
                $item->setBopisShippingcode($shipLabel);
                $item->setBopisShippingfee($approx_ship_amounts[$i]);
                $item->save();

            }
            $i++;
        }
       
        
   }
}