<?php
namespace QuiversBopis\Integration\Plugin;

use Magento\Framework\Exception\LocalizedException;
use Magento\Shipping\Model\Config;
use Magento\Framework\App\Config\ScopeConfigInterface;
use Magento\Framework\Event\ObserverInterface;
use Magento\Framework\Event\Observer;
use QuiversBopis\Integration\Model\ConfigProvider;
use QuiversBopis\Integration\LogTracking;

class CartPlugin
{
    /**
     * @var \Magento\Quote\Model\Quote
     */
    protected $quote;
    protected $addressRepository;
    const XML_QUIVERS_UPC = 'quivers/parameters/quivers_custom_upc';
    protected $request;
    protected $shippingmodelconfig;
    protected $scopeConfig;
    protected $addressFactory;
    protected $observer;
    public $configProvider;
    private $messageManager;
    private $logger;
    /**
     * Plugin constructor.
     *
     * @param \Magento\Checkout\Model\Session $checkoutSession
     */
    public function __construct(\Magento\Directory\Model\Currency $currency,
     \Magento\Framework\Message\ManagerInterface $messageManager,
      \Magento\Checkout\Model\Session $checkoutSession, 
      \Magento\Quote\Api\Data\AddressInterface $address, Config $shippingmodelconfig, Observer $observer, ScopeConfigInterface $scopeConfig, ConfigProvider $configProvider, \Magento\Framework\App\Request\Http $request)
    {
        $this->quote = $checkoutSession->getQuote();
        $this->shippingmodelconfig = $shippingmodelconfig;
        $this->_currency = $currency;
        $this->scopeConfig = $scopeConfig;
        $this->messageManager = $messageManager;
        $this->request = $request;
        $this->address = $address;
        $this->observer = $observer;
        $this->configProvider = $configProvider;
        
    }

    /**
     * afterAddProduct
     *
     * @param      $subject
     * @param      $result    Returned value from core observed method 'addProduct'
     */
    public function afterAddProduct($subject, $result)
    {
        if(!$this->configProvider->getBopisStatus()){
            return true;
        }
        $reqData = $this->request->getParams();
        $uuid = isset($_SESSION['UUID']) ? $_SESSION['UUID'] : null;
        if ($uuid != null)
        {
            $cartData = [];
            $items = $this->quote->getAllItems();
            $items2 = $this->quote->getAllItems();
            $item_array= [];
            foreach($items as $item){
                $itemData = $item->getData();
                $is_added = false;
                $not_add = false;
                foreach($items2 as $item2){
                    $itemData2 = $item2->getData();
                    if (($itemData['sku'] == $itemData2['sku']) &&
                         ($itemData['product_type'] != $itemData2['product_type']) &&
                          ($itemData['product_type'] == "configurable")){
                        array_push($item_array, $item);
                        $is_added = true;
                        break;
                    }
                    if (($itemData['sku'] == $itemData2['sku']) &&
                        ($itemData['product_type'] != $itemData2['product_type']) &&
                        ($itemData['product_type'] == "simple")){
                        $not_add = true;
                        break;
                    }

                }
                if(!$is_added && !$not_add){
                    array_push($item_array, $item);
                }
            }
            $store = isset($reqData['store'])?json_decode($reqData['store'], true):null;
            $address = isset($store['address'])?$store['address']:null;
            try
            {
                $storeId = null;
                foreach ($item_array as $item)
                {
                    $itemupc = $this->configProvider->getUPC($item);
                    $searchtermarray = [$item->getsku(),$itemupc] ;
                    $searchterm = $reqData['searchterm'];
                    if ($reqData['config-searchterm'] != ""){
                        $searchterm = $reqData['config-searchterm'];
                    }
                    if ($searchterm !=null && in_array($searchterm,$searchtermarray))
                    {
                        $cartItems = [
                            'uuid' => $uuid, 
                            'id' => $item->getProductId() ,
                            'sku' => $item->getSku() ,
                            'userlocation'=>$reqData['userPostalcode'],
                            'businessRefId' => isset($store['businessRefId'])?$store['businessRefId']:null,
                            'searchterm' => $searchterm, 
                            'businessName' => isset($store['businessName'])?$store['businessName']:null, 
                            'city' =>  isset($store['businessRefId'])?$address['city']:null, 
                            'storepostalcode' => isset($address['postalCode'])?$address['postalCode']:null,
                            'state' => isset($address['region']['abbreviation'])?$address['region']['abbreviation']:null,
                            'country' => isset($address['country']['abbreviations']['two'])?$address['country']['abbreviations']['two']:null, 
                            'curbsidepick' => isset($reqData['shipping']) && $reqData['shipping']== 2 ? true : false, 
                            'instorepickup' => isset($reqData['shipping']) && $reqData['shipping'] == 1 ? true : false,
                            'bopis_item' => isset($store['item']) && $store['item']!=null? $store['item']:null
                        ];
                        $storeId = isset($store['businessRefId'])?$store['businessRefId']:null;
                        $item->setBopisdata(json_encode($cartItems));
                        $item->setBopisShippingcode(null);
                        $item->setBopisShippingfee(null);
                    }
                    $bopisdata = json_decode($item->getBopisdata());
                }
                $this->configProvider->getShippingCarrier($storeId);

            }
            catch(\Exception $e)
            {
                $this->messageManager->addErrorMessage($e->getMessage());
            }
        }
        else
        {
            $msg ="Something went wrong ".$this->configProvider->errormessagedefault;
            throw new \Exception($msg);
        }
    }

}
?>
