<?php

namespace QuiversBopis\Integration\Plugin;

use Magento\Sales\Api\Data\OrderItemExtensionFactory;
use Magento\Sales\Api\Data\OrderItemExtensionInterface;
use Magento\Sales\Api\Data\OrderItemInterface;
use Magento\Sales\Api\Data\OrderItemSearchResultInterface;
use Magento\Sales\Api\OrderItemRepositoryInterface;

/**
 * Class OrderItemRepositoryPlugin
 */
class OrderItemRepositoryPlugin
{
    /**
     * Order feedback field name
     */
    const FIELD_NAME = 'bopisdata';
    const SHIPPING_CODE_NAME = 'bopis_shippingcode';
    const SHIPPING_CODE_FEE = 'bopis_shippingfee';


    /**
     * Order Extension Attributes Factory
     *
     * @var OrderItemExtensionFactory
     */
    protected $extensionFactory;

    /**
     * OrderItemRepositoryPlugin constructor
     *
     * @param OrderItemExtensionFactory $extensionFactory
     */
    public function __construct(OrderItemExtensionFactory $extensionFactory)
    {
        $this->extensionFactory = $extensionFactory;
    }

    /**
     *
     * @param OrderItemRepositoryInterface $subject
     * @param OrderItemSearchResultInterface $searchResult
     *
     * @return OrderItemSearchResultInterface
     */
    public function afterGetList(OrderItemRepositoryInterface $subject, OrderItemSearchResultInterface $searchResult)
    {   
        $orderItems = $searchResult->getItems();
        foreach ($orderItems as $item) {
            $bopisdata = $item->getData(self::FIELD_NAME);
            $bopiscode= $item->getData(self::SHIPPING_CODE_NAME);
            $bopisfee = $item->getData(self::SHIPPING_CODE_FEE);
            $extensionAttributes = $item->getExtensionAttributes();
            $extensionAttributes = $extensionAttributes ? $extensionAttributes : $this->extensionFactory->create();
            $extensionAttributes->setBopisdata($bopisdata);
            $extensionAttributes->setBopisShippingcode($bopiscode);
            $extensionAttributes->setBopisShippingfee($bopisfee);
            $item->setExtensionAttributes($extensionAttributes);
        }

        return $searchResult;
    }
}
